/*
** Caution: This program can only be compiled correctly with a compiler supporting
** OpenMP 4.0. Gcc supports OpenMP 4.0 beginning with version 4.9!
*/
 
#include <iostream>
#include <iomanip>
#include <math.h>
#include <sys/time.h>

using namespace std;


/*
** Auxiliary function: returns the current time in seconds as a floating point number.
*/
double getTime()
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return tv.tv_sec + tv.tv_usec * 0.000001;
}

/*
** Auxiliary function: computes for a given amount of time.
*/
void compute(double computeTime)
{
	volatile static double glob = 0;
	int i;
	double start = getTime();
	while (getTime() - start < computeTime) {
		for (i=0; i<100000; i++)
			glob += sin(i/10000.0);
	}
}

double computeA()
{
	double res;
	cout << "computeA started\n";
	compute(1.0);
	cout << "computeA ended\n";
	return 0.13456;
}

double computeB()
{
	double res;
	cout << "computeB started\n";
	compute(1.0);
	cout << "computeB ended\n";
	return 1.1;
}

double computeCfromAandB(double a, double b)
{
	cout << "computeCfromAandB started\n";
	compute(1.0);
	cout << "computeCfromAandB ended\n";
	return a + b;
}

double computeBagain()
{
	cout << "computeBagain started\n";
	compute(1.0);
	cout << "computeBagain ended\n";
	return 999;
}

int main(int argc, char **argv)
{
	double a, b, c;
	double start;

	start = getTime();
#pragma omp parallel
#pragma omp single
	{
#pragma omp task shared(a) depend(out: a)
		a = computeA();
#pragma omp task shared(b) depend(out: b)
		b = computeB();
#pragma omp task shared(a,b,c) depend(in: a,b)
		c = computeCfromAandB(a, b);
#pragma omp task shared(b) depend(out: b)
		b = computeBagain();
	}
	cout << "Result = " << c << " Time: " << (getTime()-start) << "\n";
	return 0;
}
